import simplejson as json

from alice.commons.base import Base
from alice.config.message_template import MSG_BAD_START, MSG_NO_TECH_REVIEW, MSG_AUTO_CLOSE, MSG_OPENED_TO_MAIN_BRANCH, \
    MSG_OPENED_TO_PREVENTED_BRANCH, SPECIAL_COMMENT, GENERAL_COMMENT, MSG_RELEASE_PREPARATION, \
    MSG_SENSITIVE_FILE_TOUCHED, MSG_QA_SIGN_OFF, MSG_CODE_CHANNEL, MSG_GUIDELINE_ON_MERGE
from alice.config.message_template import MSG_NO_PRODUCT_REVIEW
from alice.helper.constants import THUMBS_UP_ICON
from alice.helper.github_helper import GithubHelper
from alice.helper.github_helper import PRFilesNotFoundException
from alice.helper.log_utils import LOG
from alice.helper.slack_helper import SlackHelper


#app = Flask(__name__)


class Actor(Base):
    
    def __init__(self, pr):
        self.pr = pr
        self.github = GithubHelper(self.pr)
        self.slack = SlackHelper(self.pr.config)
        self.change_requires_product_plus1 = False
        self.is_product_plus1 = False
        self.sensitive_file_touched = {}
        self.base_branch = self.pr.base_branch
        self.head_branch = self.pr.head_branch
        self.created_by = self.pr.config.getSlackName(self.pr.opened_by)
        self.merged_by = self.pr.config.getSlackName(self.pr.merged_by)
        self.sensitive_file_touched, self.change_requires_product_plus1 = self.parse_files_and_set_flags()


    def parse_files_and_set_flags(self):
        """
        Parse payload and keep important flags ready to be used in checks
        :return: sensitive_file_touched (dict of file name with boolean value)
        :return: change_requires_product_plus1 (boolean)
        """
        change_requires_product_plus1 = False
        sensitive_file_touched = {}
        try:
            files_contents = self.github.get_files()
            LOG.info("**** Reading files ****")
            for item in files_contents:
                file_path = item["filename"]
                if any(x in str(file_path) for x in self.pr.config.sensitiveFiles):
                    sensitive_file_touched["is_found"] = True
                    sensitive_file_touched["file_name"] = str(file_path)
                if item["filename"].find(self.pr.config.productPlusRequiredDirPattern) != -1:
                    LOG.info("product change found marking ui_change to True")
                    change_requires_product_plus1 = True
                    # break
        except PRFilesNotFoundException, e:
            LOG.exception(e)
        return sensitive_file_touched, change_requires_product_plus1

    def is_bad_pr(self):
        """
        parse approval's content to identify if PR is actually approved or just random approval click
        :return: bad_pr (boolean)
        """
        reviews = self.github.get_reviews()
        if 200 != reviews.status_code:
            raise Exception(reviews.content)

        bad_pr = True
        LOG.info("***** Reading Reviews *****")
        for item in json.loads(reviews.content):
            if "APPROVED" == item["state"]:
                review_comment = item["body"]
                LOG.debug("review body= %s" + review_comment)
                thumbsUpIcon = THUMBS_UP_ICON in json.dumps(review_comment)
                LOG.debug("unicode thumbsUp icon present=%s" % (thumbsUpIcon))
                if self.pr.opened_by in self.pr.config.superMembers:  # FEW FOLKS TO ALLOW TO HAVE SUPER POWER
                    LOG.debug("PR is opened by %s who is the super user of repo %s, so NO alert'"
                              % (self.pr.opened_by_slack, self.pr.repo))
                    bad_pr = False
                    break
                print "***** review_comment", review_comment
                created_by = self.pr.config.getSlackName(self.pr.opened_by)
                if item["user"]["login"] != created_by and (review_comment.find("+1") != -1 or thumbsUpIcon):
                    LOG.debug("+1 is found from reviewer=%s marking No Alert " % item["user"]["login"])
                    bad_pr = False
                    break
        return bad_pr

    def validate_tech_approval(self):
        """
        notify in channel if not tech approved
        :return: relevant response dict
        """
        if self.pr.is_merged:  # and self.head_branch in PushPayload.PROTECTED_BRANCH_LIST:  TO ENABLE
            is_bad_pr = self.is_bad_pr()
            bad_name_str = MSG_BAD_START + "@" + self.created_by
            if is_bad_pr:
                msg = MSG_NO_TECH_REVIEW.format(name=bad_name_str, pr=self.pr.link_pretty, title=self.pr.title,
                                                branch=self.pr.base_branch, team=self.pr.config.cc_tech_team)
                LOG.debug(msg)
                self.slack.postToSlack(self.pr.config.alertChannelName, msg)
                LOG.info("Bad PR={msg} repo:{repo}".format(repo=self.pr.repo, msg=is_bad_pr))
                return {"msg": "Bad PR={msg} repo:{repo}".format(repo=self.pr.repo, msg=is_bad_pr)}
            return {"msg": "PR is approved so No Alerts"}
        return {"msg": "Skipped review because its not PR merge event"}

    def validate_product_approval(self):
        """
        notify in channel if not product approved (applicable only to files/dir which require product approval)
        :return: relevant response dict
        """
        if self.pr.is_merged:
            if self.pr.opened_by in self.pr.config.superMembers:
                LOG.debug('pr_opened_by is super user of {repo} so NO alert, super_members={super_members}'
                          .format(repo=self.pr.repo, super_members=self.pr.config.superMembers))
                return {"msg": "Skipped product review because the PR=%s is by a Super User" % self.pr.link_pretty}

            if self.change_requires_product_plus1:
                comments = self.github.get_comments()
                is_product_plus1 = self.is_plus_1_in_comments(comments, self.pr.config.productTeamGithub)
                if not is_product_plus1:
                    bad_name_str = MSG_BAD_START + "@" + self.created_by
                    msg = MSG_NO_PRODUCT_REVIEW.format(name=bad_name_str, pr=self.pr.link_pretty, title=self.pr.title,
                                                    branch=self.pr.base_branch, team="".join(self.pr.config.productTeamToBeNotified))
                    LOG.debug(msg)
                    self.slack.postToSlack(self.pr.config.alertChannelName, msg)
                    LOG.info("Bad PR={msg} repo:{repo}".format(repo=self.pr.repo, msg=self.is_bad_pr))
                    return {"msg": "Bad PR={msg} repo:{repo}".format(repo=self.pr.repo, msg=self.is_bad_pr)}
                return {"msg":"Product approved so no alert, pr=%s" %self.pr.link_pretty}
            return {"msg":"Skipped product review because no changes found which requires product +1 as well, PR=%s"
                          %self.pr.link_pretty}
        return {"msg": "Skipped product review because the PR=%s is not merged" %self.pr.link_pretty}

    def is_plus_1_in_comments(self, comments, team):
        """
        parse comments and check if :+1: exists (symbol for approval)
        :param comments:
        :return: relevant response dict
        """
        is_plus_1 = False
        if self.change_requires_product_plus1:
            for comment in json.loads(comments["content"]):
                thumbsUpIcon = "\ud83d\udc4d" in json.dumps(comment["body"])
                if self.change_requires_product_plus1 and (comment["user"]["login"] in team) \
                        and (comment["body"].find("+1") != -1 or thumbsUpIcon):
                    is_plus_1 = True
                    break
        return is_plus_1

    def comment_on_pr(self):
        """
        comment on pr
        :return: relevant response dict
        """
        if self.pr.is_opened:
            if not self.pr.config.is_debug:
                if self.pr.base_branch == self.pr.config.mainBranch:
                    guideline_comment = SPECIAL_COMMENT
                    self.slack.postToSlack(self.pr.opened_by)
                else:
                    guideline_comment = GENERAL_COMMENT
                self.github.comment_pr(self.pr.comments_section, guideline_comment)
                LOG.info("**** Added Comment of dev guidelines ***")
                return {"msg": "Added Comment of dev guidelines"}
            return {"msg": "Skipped commenting because DEBUG is on "}
        return {"msg": "Skipped commenting because its not PR opened"}


    def notify_on_action(self):
        """
        notify respective folk(s) on particular action. Ex. on open notify to lead (or on merged, can configure that)
        :return: relevant response dict
        """
        desired_action = self.pr.config.actionToBeNotifiedFor
        if self.pr.action == desired_action:
            if self.pr.base_branch == self.pr.config.mainBranch:
                msg = MSG_OPENED_TO_MAIN_BRANCH.format(repo=self.pr.repo, pr_by=self.created_by,
                                                       main_branch=self.pr.config.mainBranch, title=self.pr.title,
                                                       pr=self.pr.link_pretty, action=self.pr.action)
                for person in self.pr.config.techLeadsToBeNotified:
                    self.slack.postToSlack(person, msg + MSG_RELEASE_PREPARATION)
                LOG.info("Notified to %s on action %s" % (self.pr.config.techLeadsToBeNotified, self.pr.action))
                return {"msg": "Notified to %s on action %s" % (self.pr.config.techLeadsToBeNotified, self.pr.action)}
            else:
                msg = MSG_OPENED_TO_PREVENTED_BRANCH.format(repo=self.pr.repo, pr_by=self.created_by,
                                                            base_branch=self.pr.base_branch, title=self.pr.title,
                                                            pr=self.pr.link_pretty, action=self.pr.action)
                self.slack.postToSlack(self.pr.config.personToBeNotified, msg)
                LOG.info("Notified to %s on action %s" % (self.pr.config.personToBeNotified, self.pr.action))
                return {"msg": "Notified to %s on action %s" % (self.pr.config.personToBeNotified, self.pr.action)}
        return {"msg": "Skipped notify because its (%s) not desired event '%s'" %(self.pr.action, desired_action)}


    def remind_direct_release_guideline_on_merge(self):
        """
        remind individually to follow guidelines for QA process
        :return: relevant response dict
        """
        if self.pr.is_merged:
            if self.base_branch in self.pr.config.sensitiveBranches:
                msg = MSG_GUIDELINE_ON_MERGE.format(person=self.created_by, pr=self.pr.link_pretty,
                                                    base_branch=self.pr.base_branch, title=self.pr.title,
                                                    release_notes_link=self.pr.config.release_notes_link)
                self.slack.directSlack('@' + self.created_by, msg)
                LOG.info("slacked personally to %s" % self.created_by)
                return {"msg": "slacked personally to %s" % self.created_by}
            return {"msg": "skipped slack personally because not sensitive branch"}
        return {"msg": "skipped slack personally because its not merge event" % self.created_by}


    def close_dangerous_pr(self):
        """
        close a Pull Request which is not supposed to be opened Ex. base=master head=feature
        :return: relevant response dict
        """
        if self.pr.is_opened or self.pr.is_reopened:
            master_branch = self.pr.config.mainBranch
            qa_branch = self.pr.config.testBranch
            if self.base_branch == master_branch and self.head_branch != qa_branch:
                msg = MSG_AUTO_CLOSE.format(tested_branch=qa_branch, main_branch=master_branch)
                self.github.modify_pr(msg, "closed")
                self.slack.postToSlack(self.pr.config.alertChannelName, "@" + self.created_by + ": " + msg)
                LOG.info("closed dangerous PR %s" % self.pr.link_pretty)
                return {"msg": "closed dangerous PR %s" % self.pr.link_pretty}
            return {"msg": "skipped closing PR because not raised to mainBranch %s" % self.pr.link_pretty}
        return {"msg": "skipped closing PR because not a opened PR"}

    def notify_if_sensitive_modified(self):
        """
        check & notify devOps team if any sensitive files are touched
        :return: relevant response dict
        """
        if self.pr.is_merged:
            if self.sensitive_file_touched.get("is_found"):
                msg = MSG_SENSITIVE_FILE_TOUCHED.format(
                    notify_folks=self.pr.config.devOpsTeamToBeNotified, file=self.sensitive_file_touched["file_name"],
                    pr=self.pr.link_pretty, pr_by=self.created_by, pr_number=self.pr.number)
                self.slack.postToSlack(self.pr.config.alertChannelName, msg)
                LOG.info("informed %s because sensitive files are touched in pr=%s" %
                         (self.pr.config.devOpsTeamToBeNotified, self.pr.link_pretty))
                return {"msg": "informed %s because sensitive files are touched" % self.pr.config.devOpsTeamToBeNotified}
            return {"msg": "Skipped sensitive files alerts because no sensitive file being touched"}
        return {
            "msg": "Skipped sensitive files alerts because its not PR merge event %s" % self.pr.config.devOpsTeamToBeNotified}


    def notify_qa_sign(self):
        if self.pr.is_merged and self.pr.base_branch == self.pr.config.mainBranch \
                and self.pr.head_branch == self.pr.config.testBranch:
            msg = MSG_QA_SIGN_OFF.format(person=self.pr.config.personToBeNotified, pr=self.pr.link_pretty,
                                         dev_ops_team=self.pr.config.devOpsTeamToBeNotified,
                                         tech_team=self.pr.config.techLeadsToBeNotified)

            self.slack.postToSlack(self.pr.config.alertChannelName, msg,
                               data=self.slack.getBot(self.pr.config.alertChannelName, self.merged_by))
        """ for bot """
        # write_to_file_from_top(release_freeze_details_path, ":clubs:" +
        #                        str(datetime.now(pytz.timezone('Asia/Calcutta')).strftime(
        #                            '%B %d,%Y at %I.%M %p')) + " with <" + self.pr.link_pretty + "|master> code")  # on:" + str(datetime.datetime.now().strftime('%B %d, %Y @ %I.%M%p'))
        # clear_file(code_freeze_details_path)

    def notify_channel_on_merge(self):
        """
        pass entry in fixed channel for all code merge details (merged to any of sensitive branch)
        :return: relevant response dict
        """
        if self.pr.is_merged:
            # print "**** Repo=" + repo + ", new merge came to " + base_branch + " set trace to " + code_merge_channel + " channel"
            msg = MSG_CODE_CHANNEL.format(title=self.pr.title, desc=self.pr.description, pr=self.pr.link,
                                          head_branch=self.pr.head_branch, base_branch=self.pr.base_branch,
                                          pr_by=self.created_by, merge_by=self.merged_by)
            self.slack.postToSlack(self.pr.config.alertChannelName, msg)
            LOG.info("informed %s because pr=%s is merged into sensitive branch=%s" %
                     (self.pr.config.alertChannelName, self.pr.link_pretty, self.pr.base_branch))
            return {"msg":"informed %s because pr=%s is merged into sensitive branch=%s" %
                     (self.pr.config.alertChannelName, self.pr.link_pretty, self.pr.base_branch)}
        return {"msg", "Skipped posting to code channel because '%s' is not merge event" %self.pr.action}






import logging
import os
from alice.helper.file_utils import get_dict_from_config_file

LOG = logging.getLogger()
handler = logging.StreamHandler()
formatter = logging.Formatter(
    '%(asctime)s %(levelname)s: %(message)s '
    '[in %(pathname)s:%(lineno)d]')
handler.setFormatter(formatter)
LOG.addHandler(handler)
config_file = os.environ["config"]
config = get_dict_from_config_file(config_file)
debug = config.get('debug', False)
if debug:
    LOG.setLevel(logging.DEBUG)
else:
    LOG.setLevel(logging.INFO)


from alice.main.actor import Actor


class Checks(object):
    def __init__(self, pr_payload):
        self.actor = Actor(pr=pr_payload)


    def tech_review(self):
        """
        checks for +1 in review approved
        :return:
        """
        return self.actor.validate_tech_approval()


    def comment_guidelines(self):
        """
        add comment on opened PR
        :return:
        """
        return self.actor.comment_on_pr()

    def notify_channel_on_merge(self):
        """
        store merged PR data to respective channel
        :return:
        """
        return self.actor.notify_channel_on_merge()

    def notify_lead_on_given_action(self):
        """
        keep lead posted on particular action on sensitive branch
        :return:
        """
        # if self.pr.is_opened:
        return self.actor.notify_on_action()

    def remind_direct_release_guideline_on_merge(self):
        """
        remind the member who had raised PR to any of the sensitive branch to follow specific release guideline
        :return:
        """
        return self.actor.remind_direct_release_guideline_on_merge()

    def close_dangerous_pr(self):
        """
        senses
        :return:
        """
        return self.actor.close_dangerous_pr()

    def notify_on_sensitive_files_touched(self):
        return self.actor.notify_if_sensitive_modified()

    def product_review(self):
        return self.actor.validate_product_approval()

    """ TO DO """
    # def notify_qa_sign_off(self):
    #     return self.actor.notify_qa_sign()
    #
    # def notify_to_add_release_notes_for_next_release(self):
    #     pass
    #
    # def announce_code_freeze(self):
    #     pass
    #
    # def ci_lint_checker(self):
    #     pass
    #
    # def ci_unit_tests(self):
    #     pass




import simplejson as json
import pkg_resources
import os
from alice.helper.api_manager import ApiManager
from alice.helper.constants import API_GITHUB_MEMBERS_LIST, API_GITHUB_USERS, SLACK_USER_LIST
from alice.helper.file_utils import get_dict_from_config_file
git_mappings = {}
slack_mappings = {}


class CommonUtils(object):
    config_file = os.environ["config"]
    config = get_dict_from_config_file(config_file)
    GIT_TOKEN = config.get('tokens').get("github")
    SLACK_TOKEN = config.get('tokens').get("slack")
    organisation = config.get('organisation')

    @staticmethod
    def getGithubUsers():
        if git_mappings:
            return git_mappings
        users = []
        page = 1
        while True:
            member_api = "%s%s" % (API_GITHUB_MEMBERS_LIST.format(org=CommonUtils.organisation), page)
            response = ApiManager.get(member_api, headers={"Authorization": "token " + CommonUtils.GIT_TOKEN})
            if not response:
                break
            users += json.loads(response["content"])
            page += 1

        for item in users:
            user_name = item["login"]
            ApiManager.get(API_GITHUB_USERS + "/" + user_name)
            git_mappings[item["login"]] = item["login"]
        return git_mappings

    @staticmethod
    def getSlackUsers():
        if slack_mappings:
            return slack_mappings
        response = ApiManager.get(SLACK_USER_LIST + CommonUtils.SLACK_TOKEN, headers={})
        users = json.loads(response["content"])

        for item in users["members"]:
            slack_mappings[item["name"]] = item["profile"].get("email","bot@gmail.com")
        print slack_mappings

    @staticmethod
    def getSlackNicksFromGitNicks(key):
        if key in git_mappings:
            return git_mappings[key]
        return key

    @staticmethod
    def readResourceJson(module, path):
        json_string = CommonUtils.readResourceString(module, path)
        return json.loads(json_string)

    @staticmethod
    def readResourceString(module, path):
        return pkg_resources.resource_string(module, path)





from alice.helper.colors import Blinking_Colors

MSG_BAD_START = "Very Bad "
MSG_NO_TECH_REVIEW = "{name} :rage: <{pr}|{title}> is merged into `{branch}` without a \"Tech +1\"," \
                     " soon these kind of requests will be automatically reverted CC: {team}"
MSG_NO_PRODUCT_REVIEW = "{name} :rage: <{pr}|{title}>  is merged into `{branch}` without \"Product +1\"," \
                        " soon these kind of requests will be automatically reverted CC: {team}"

MSG_CODE_CHANNEL = "Title=\"{title}\", Description=\"{desc}\" \nPR: {pr}\n from {head_branch} into `{base_branch}`" \
                   " By: *{pr_by}*, mergedBy: {merge_by}\n"

MSG_RELEASE_PREPARATION = "\n Please review and approve with +1, Release preparation starts..."

MSG_GUIDELINE_ON_MERGE = "Hi @{person}: you have merged <{pr}|{title}> into {base_branch}\n now, be nice & mention it " \
                         "in <{release_notes_link}|Release Notes> for getting it `QAed and released` " \
                         "under Sheet name as respective Date"

MSG_AUTO_CLOSE = "alice have auto-closed it because she sensed it an accidental PR (only \"{tested_branch}\" " \
                 "can be merged to \"{main_branch}\") \n Alice is smart! Be like Alice!"

GENERAL_COMMENT = {"body": "Did you remember to?\n"
                           "- [ ] Add Test Case(s)"
                           "- [ ] P0 Tests Executed EndToEnd?"
                   }

MSG_OPENED_TO_MAIN_BRANCH = "{repo} repo:: <{pr}|{title}> is {action} to `{main_branch}` by:*{pr_by}* " \
                            "\n Please review as Release preparation starts now ..."
MSG_OPENED_TO_PREVENTED_BRANCH = "{repo} repo:: <{pr}|{title}> is {action} to `{base_branch}` by:*{pr_by}* "

MSG_QA_SIGN_OFF = "<@{person}>  QA passed :+1: {main_branch} is updated <{pr}|Details here> Awaiting your go ahead." \
                  " \n cc: {dev_ops_team} {tech_team} "
MSG_SENSITIVE_FILE_TOUCHED = "{notify_folks} {file}  is modified in <{pr}|{pr_number}> by @{pr_by}"

SPECIAL_COMMENT = {
    "body": "**Attention!** \n Release Checklist\n"
            "- [ ] JS version Update? check index.html\n"
            "- [ ] No code/PR to be reverted? check release notes\n"
            "- [ ] Unit Tests Passed?\n"
            "- [ ] Api Tests passed?\n"
            "- [ ] QA report linked?\n"
            "- [ ] Release Notes linked?"
}

DOC_CHECK_NOT_FOUND = "\"{check_name}\" Check is not found (please cross check the checks defined in config file). \n" \
                      "If you wish to implement this new check yourself, please read here: " + Blinking_Colors.OKBLUE + \
                      "{doc_link}" + Blinking_Colors.END
ISSUE_FOUND = "\nOo this looks like i've messed up, Please raise this as issue at: " + Blinking_Colors.OKBLUE \
              + "{issue_link}" + Blinking_Colors.END + " will try to help you asap"



import os
from alice.helper.file_utils import get_dict_from_config_file, get_dict_from_yaml
from alice.helper.log_utils import LOG


class ConfigProvider(object):
    # __metaclass__ = SingletonMetaClass

    def __init__(self, repo):
        config_file = os.environ["config"]
        LOG.info("********** config file=" + config_file)
        # absolute path to keep file anywhere
        self.config = get_dict_from_config_file(config_file)
        self.repo_name = repo

    def __str__(self):
        return "Alice - Common Config Provider"

    @property
    def organisation(self):
        return self.config.get('organisation')

    @property
    def githubToken(self):
        return self.config.get('tokens').get("github")

    @property
    def slackToken(self):
        return self.config.get('tokens').get("slack")

    @property
    def is_debug(self):
        return self.config.get("debug", False)

    @property
    def repo(self):
        return self.config.get("repo").get(self.repo_name, {})

    @property
    def sensitiveBranches(self):
        return self.repo.get('sensitive_branches')

    @property
    def sensitiveFiles(self):
        return self.repo.get("sensitive_files")

    @property
    def branchListToBeNotifiedFor(self):
        return self.repo.get('notify_direct', {}).get('branch_list_to_be_notified')

    @property
    def actionToBeNotifiedFor(self):
        return self.repo.get('notify_direct', {}).get('action_to_be_notified_on', "opened")

    @property
    def superMembers(self):
        return self.repo.get('super_git_members')

    @property
    def mainBranch(self):
        return self.repo.get('main_branch')

    @property
    def testBranch(self):
        return self.repo.get('test_branch')

    @property
    def debug_folks(self):
        return self.config.get('debug_alice', {}).get('debug_folks')

    @property
    def debug_channel(self):
        return self.config.get('debug_alice', {}).get('debug_channel')

    @property
    def alertChannelName(self):
        if self.is_debug:
            return self.debug_channel
        return self.repo.get('alert_channel')

    @property
    def cc_tech_team(self):
        if self.is_debug:
            return self.debug_folks
        return self.repo.get('cc_members')

    @property
    def codeChannelName(self):
        if self.is_debug:
            return self.debug_channel
        return self.repo.get('code_channel')

    @property
    def personToBeNotified(self):
        if self.is_debug:
            return self.debug_folks
        return self.repo.get('notify_direct', {}).get('person_to_be_notified')

    @property
    def techLeadsToBeNotified(self):
        if self.is_debug:
            return self.debug_folks
        return self.repo.get('notify_direct', {}).get('tech_leads_to_be_notified_on_release_freeze')

    @property
    def productTeamToBeNotified(self):
        if self.is_debug:
            return self.debug_folks
        return self.repo.get('product_team')

    @property
    def productTeamGithub(self):
        return self.repo.get('product_team_github_names')

    @property
    def productPlusRequiredDirPattern(self):
        return self.repo.get('product_plus_required_dir_pattern')

    @property
    def devOpsTeamToBeNotified(self):
        if self.is_debug:
            return self.debug_folks
        return self.config.get("dev_ops_team", "")

    @property
    def devOpsTeamMembers(self):
        return self.config.get("dev_ops_team", "")

    @property
    def checks(self):
        return self.repo.get("checks",[])

    @property
    def release_notes_link(self):
        return self.config.get("release_notes_link")

    def getSlackName(self, github_name):
        return self.config.get('user_map',{}).get(github_name, github_name)



import json
import requests
from alice.helper.api_manager import ApiManager
from alice.helper.constants  import API_GITHUB_REVIEW_ACCEPT_KEY, EP_REVIEWS, API_GITHUB_REPO_MEMBER, API_GITHUB_ISSUES, \
    EP_COMMENTS
from alice.helper.decorators.retry import Retry
from alice.helper.log_utils import LOG


class PRFilesNotFoundException(Exception):
    def __init__(self, pr_response):
        self.pr_response = pr_response
        super(PRFilesNotFoundException, self).__init__(str(self.pr_response))


class PRContentNotFoundException(Exception):
    def __init__(self, pr_response):
        self.pr_response = pr_response
        super(PRContentNotFoundException, self).__init__(str(self.pr_response))


class GithubHelper(object):

    def __init__(self, pr):
        self.GITHUB_TOKEN = pr.config.githubToken
        self.pr_api_link =  pr.link
        self.headers = {"Authorization": "token " + self.GITHUB_TOKEN}
        self.pr = pr

        url = API_GITHUB_REPO_MEMBER.format(org=self.pr.config.organisation, repo=self.pr.repo)
        response = ApiManager.get(url=url, headers=self.headers)
        if response["status_code"] != 200:
            raise Exception(response["content"], "Please check the provided Github Token, "
                                                 "either user doesn't have permission to"
                                                 " the organisation or the repository")

    def comment_pr(self, comment_section, comment):
        resp = requests.post(comment_section, headers=self.headers,
                             data=json.dumps(comment))
        LOG.debug(resp.content)

    def modify_pr(self, msg, state):
        data = {
            "title": msg,
            "state": state
        }
        resp = requests.post(self.pr_api_link, json.dumps(data), headers=self.headers)
        LOG.debug(resp.content)

    def get_reviews(self):
        url = self.pr_api_link + "/" + EP_REVIEWS
        self.headers["Accept"] = API_GITHUB_REVIEW_ACCEPT_KEY
        return requests.get(url, headers=self.headers)

    def get_files_requests(self):
        url = self.pr_api_link + "/files"
        files = ApiManager.get(url, headers=self.headers)
        return files["response"].json()

    def is_pr_file_content_available(self, response):
        return not (isinstance(response, dict) and 'message' in response and response['message'] == "Not Found")

    @Retry(PRFilesNotFoundException, max_retries=20)
    def get_files(self):
        files_content = self.get_files_requests()
        if not self.is_pr_file_content_available(files_content):
            raise PRFilesNotFoundException(files_content)
        return files_content

    @Retry(PRContentNotFoundException, max_retries=3)
    def get_comments(self):
        comments_end_point = API_GITHUB_ISSUES.format(org=self.pr.config.organisation, repo=self.pr.repo) \
                             + "/" + str(self.pr.number) + "/" + EP_COMMENTS
        return ApiManager.get(comments_end_point, self.headers)




from alice.helper.constants import SLACK_ICON
from slacker import Slacker
from alice.helper.log_utils import LOG


class SlackHelper(object):
    def __init__(self, config):
        self.config = config
        self.slack = Slacker(self.config.slackToken)
        self.icon = SLACK_ICON

    def postToSlack(self, channel, msg=None, *args, **kwargs):
        LOG.info("\n************** NOTIFYING ******************\n"
                    "**************  %s      *************\n"
                    "Message= %s\n"
                    "******************************************* " % (channel, msg))
        self.slack.chat.post_message(channel=channel, text=msg, icon_url=self.icon, username="Alice", *args, **kwargs)

    def directSlack(self, person, msg=None, *args, **kwargs):
        if self.config.is_debug:
            person = self.config.debug_folks
        LOG.info("\n************** NOTIFYING ******************\n"
                    "**************  %s      *************\n"
                    "Message= %s\n"
                    "******************************************* " % (person, msg))
        self.slack.chat.post_message(channel=person, text=msg, icon_url=self.icon, username="Alice", *args, **kwargs)

    def getBot(self, channel, user):
        icon_url = "https://cloud.githubusercontent.com/assets/12966925/25274594/528675da-26ae-11e7-8331-25f25c41b75d.png"
        return {"username": user, "icon_url": icon_url}



"""" API ROUTES """
SLACK_USER_LIST = "https://slack.com/api/users.list?token="
API_GITHUB_USERS = "https://api.github.com/users/"
API_GITHUB_MEMBERS_LIST = "https://api.github.com/orgs/{org}/members?page="
API_GITHUB_MEMBERS = "https://api.github.com/orgs/{org}/members"
API_GITHUB_REPO_MEMBER = "https://api.github.com/repos/{org}/{repo}"
API_GITHUB_REVIEW_ACCEPT_KEY = "application/vnd.github.black-cat-preview+json"
API_GITHUB_ISSUES = "https://api.github.com/repos/{org}/{repo}/issues"

EP_REVIEWS = "reviews"
EP_COMMENTS = "comments"
STATE_OPEN = "open"
STATE_CLOSED = "closed"


""" docs link """
EXTEND_ALICE= "https://github.com/moengage/alice/blob/master/docs/extend_alice.md"
config_file="https://github.com/moengage/alice/blob/master/docs/setup_config.md"
ISSUE_LINK= "https://github.com/moengage/alice/issues/new"


""" other constants """
THUMBS_UP_ICON = "\ud83d\udc4d"
SLACK_ICON = "https://cloud.githubusercontent.com/assets/12966925/25132384/021095fc-2467-11e7-95c0-78917bf4d52a.png"
bot_name = "alice"



from alice.main.checks import Checks


class CheckImpl(Checks):

    def __init__(self, push_payload_parser):
        super(CheckImpl, self).__init__(push_payload_parser)
        self.pr = push_payload_parser


    """
    Implement your check here, will be automatically called

    To get pull request related data, access with self.pr Ex.
        self.pr.repo
        self.pr.number
        self.pr.opened_by
        self.pr.merged_by
        self.pr.merged_by_slack
        self.pr.opened_by_slack
        self.pr.link_pretty   -> readable link
        self.pr.link          -> api link
        self.pr.action
        self.pr.is_merged
        self.pr.is_opened
        self.pr.is_reopened
        self.pr.base_branch
        self.pr.head_branch
        self.pr.comments_section
        self.pr.is_sensitive_branch
        self.pr.title
        self.pr.description


   To get inputs from config file, access with self.pr.config
       self.pr.config.organisation
       self.pr.config.githubToken
       self.pr.config.slackToken
       self.pr.config.is_debug
       self.pr.config.repo
       self.pr.config.sensitiveBranches
       self.pr.config.sensitiveFiles
       self.pr.config.branchListToBeNotifiedFor
       self.pr.config.actionToBeNotifiedFor
       self.pr.config.whiteListedMembers
       self.pr.config.superMembers
       self.pr.config.mainBranch
       self.pr.config.testBranch
       self.pr.config.debug_folks
       self.pr.config.debug_channel
       self.pr.config.alertChannelName
       self.pr.config.codeChannelName
       self.pr.config.personToBeNotified
       self.pr.config.techLeadsToBeNotified
       self.pr.config.productPlusRequiredDirPattern
       self.pr.config.devOpsTeam
       self.pr.config.checks
       self.pr.config.getSlackName(github_name)

    """




from flask import Flask, request, jsonify, abort
import simplejson as json
from alice.main.runner import RunChecks
from alice.helper.log_utils import LOG

app = Flask(__name__)

@app.route("/alice", methods=['POST'])
def alice():
    if request.method != 'POST':
        abort(501)
    payload = request.get_data()
    data = json.loads(unicode(payload, errors='replace'), strict=False)
    merge_correctness = RunChecks().run_checks(request, data)
    return jsonify(merge_correctness)


@app.route("/", methods=['GET', 'POST'])
def home():
    return "************ Welcome to the world of Alice ***********"


@app.before_first_request
def setup_logging():
    if not app.debug:
        LOG.debug('************ log from setup_config *********')




import requests


class ApiManager(object):
    @staticmethod
    def get(url, headers, data=None):
        response = requests.get(url, headers=headers, data=data)
        return {"status_code": response.status_code, "content": response.content, "response": response}


    @staticmethod
    def post(url, headers, data=None):
        pass






import traceback

from alice.commons.base import PushPayloadParser
from alice.config.message_template import ISSUE_FOUND, DOC_CHECK_NOT_FOUND
from alice.helper.constants import ISSUE_LINK, EXTEND_ALICE
from alice.helper.log_utils import LOG
from alice.checker_impl import CheckImpl

class RunChecks(object):
    def execute_check(self, ci, check):
        LOG.debug("************* Starting check=%s *****************" % check)
        response = getattr(ci, check)()
        LOG.debug("for check= %s, response= %s"%(check, response))

    def run_checks(self, request, data):
        ci = CheckImpl(PushPayloadParser(request, payload=data))
        response = {}
        checks = ci.pr.config.checks
        if ci.pr.is_sensitive_branch:
            if len(checks) == 0:
                import inspect
                method_names = [attr for attr in dir(ci) if inspect.ismethod(getattr(ci, attr))]
                for check in method_names:
                    if check != "__init__":
                        self.execute_check(ci, check)
            else:
                try:
                    for check in checks:
                        try:
                            self.execute_check(ci, check)
                        except AttributeError:
                            LOG.debug("Exception in Run Checks", exc_info=traceback)
                            raise CheckNotFoundException(check)
                except Exception, e:
                    LOG.debug("Exception in Run Checks", exc_info=traceback)
                    if 'invalid_auth' not in e:
                        raise Exception(str(e) + ISSUE_FOUND.format(issue_link=ISSUE_LINK))
            return response
        LOG.info("skipped because '%s' is not sensitive branch" % ci.base_branch)
        return {"msg": "skipped because '%s' is not sensitive branch" % ci.base_branch}


class CheckNotFoundException(Exception):
    def __init__(self, method_name):
        super(CheckNotFoundException, self).__init__(DOC_CHECK_NOT_FOUND.format(check_name=method_name,
                                                                                doc_link=EXTEND_ALICE))






from alice.config.config_provider import ConfigProvider
from alice.helper.common_utils import CommonUtils
from alice.helper.log_utils import LOG


class Base(object):
    pass


class PushPayloadParser(Base):

    def __init__(self, request, payload):
        self.request = request
        self.payload = payload
        self.pr = payload["pull_request"]
        LOG.debug("Repo=" + self.repo)
        self.config = ConfigProvider(self.repo)

    @property
    def repo(self):
        return self.payload["repository"]["name"]

    @property
    def number(self):
        return self.payload["number"]

    @property
    def opened_by(self):
        return self.pr["user"]["login"]

    @property
    def merged_by(self):
        return self.pr["merged_by"]["login"]

    @property
    def merged_by_slack(self):
        return CommonUtils.getSlackNicksFromGitNicks(self.merged_by)

    @property
    def opened_by_slack(self):
        return CommonUtils.getSlackNicksFromGitNicks(self.opened_by)

    @property
    def link_pretty(self):
        return self.pr["html_url"]

    @property
    def link(self):
        return self.pr["url"]

    @property
    def is_merged(self):
        return self.pr["merged"]

    @property
    def action(self):
        return self.payload["action"]

    @property
    def is_opened(self):
        return self.action == "opened"

    @property
    def is_reopened(self):
        return self.action == "reopened"

    @property
    def base_branch(self):
        return self.pr["base"]["ref"]

    @property
    def head_branch(self):
        return self.pr["head"]["ref"]

    @property
    def comments_section(self):
        return self.pr["_links"]["comments"]["href"]

    @property
    def is_sensitive_branch(self):
        return self.base_branch in self.config.sensitiveBranches

    @property
    def title(self):
        return self.pr["title"]

    @property
    def description(self):
        return self.pr["body"]




import os
import yaml
import simplejson as json


def get_dict_from_config_file(file_path):
    if file_path.endswith(".yaml"):
        data = get_dict_from_yaml(file_path)
    elif file_path.endswith(".json"):
        with open(file_path) as data_file:
            data = json.load(data_file)
    else:
        raise Exception("The config file should be either yaml or json type")
    return data


def get_dict_from_yaml(file_path):
    data = {}
    with open(file_path, 'r') as stream:
        try:
            data = yaml.load(stream)
        except yaml.YAMLError as exc:
            print(exc)
    return data


def write_to_file_from_top(file_path, msg):
    dir_path = file_path.rpartition("/")[0]
    try:
        os.makedirs(dir_path)
    except Exception:
        if not os.path.isdir(dir_path):
            raise
    with open(file_path, "r+") as f:
        first_line = f.readline()
        lines = f.readlines()
        f.seek(0)
        f.write(msg)
        f.write("\n" + first_line)
        f.writelines(lines)
        f.close()


def write_to_file(file_path, msg):
    dir_path = file_path.rpartition("/")[0]  # partitions from last occurence
    try:
        os.makedirs(dir_path)
    except Exception:
        if not os.path.isdir(dir_path):
            raise
    with open(file_path, "a+") as f:
        f.write(msg + '\n')
        f.close()


def clear_file(_file):
    open(_file, 'w').close()


from setuptools import setup, find_packages

with open('README.rst') as f:
    long_description = f.read()

with open('VERSION') as f:
    package_version = f.read()

with open('requirements.txt') as f:
    dependencies = []
    for line in f:
        line = line.strip()
        dependencies.append(line)

setup(
    name='alice-core',
    version=package_version,
    license='MIT',
    author='Pooja Shah',
    author_email='writeback2pooja@gmail.com',
    long_description=long_description,
    packages=find_packages(exclude=['build', 'dist']),
    url='https://github.com/moengage/alice',
    include_package_data=True,
    zip_safe=False,
    # package_data={'': ['VERSION', 'changes.md']},
    install_requires=dependencies
   )







import sys
import time
from functools import wraps
import logging

class Retry(object):
    logging.basicConfig(level=logging.INFO)
    logger = logging.getLogger(__name__)

    def __init__(self, exceptions_to_catch, max_retries=2, default_value=None, after=0):
        self.max_retries = max_retries
        self.default_value = default_value
        self.exceptions_to_catch = exceptions_to_catch if exceptions_to_catch else Exception
        self.after = float(after)


    def __call__(self, func):
        @wraps(func)
        def retries(*args, **kwargs):
            return_value = self.default_value
            for i in range(self.max_retries):
                try:
                    return func(*args, **kwargs)
                except self.exceptions_to_catch:
                    if i == self.max_retries - 1 and self.default_value is None:
                        raise sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2]
                    time.sleep(self.after)
                    #self.logger.info(task=func.__name__, message="retrying function")
                    # logger.debug(task=func.__name__, message="retrying function",
                    #                retry_cause=CommonUtils.view_traceback())
            return return_value

        return retries
